/*
   decode.h

   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: decode.h,v 1.1.1.1 2004/12/08 02:44:16 jleplast Exp $ */

#ifndef __MP_DECODE_H__
#define __MP_DECODE_H__

#include <pcap.h>

typedef struct {
  struct {			/* proto - the only thing we need here */
    u_short proto;
  } ether;

  int is_ip;
  struct ip_info {
    u_long sip;			/* Source IP */
    u_long dip;			/* Dest IP */

    u_char ver;			/* version */
    u_char hl;			/* header length */
    u_char tos;			/* Type of service. Usually 0 */
    u_short len;			/* Length of packet */
    u_short id;			/* Packet ID */
    int off;			/* offset separated from flags*/
    int flags;			/* flags separated from offset*/
    int proto;			/* TCP/UDP/ICMP/... */
    u_short csum;		/* checksum */
    int ttl;
    u_char df;			/* don't fragment */
    u_char mf;			/* multiply fragments */
    u_char ub;			/* unused bit; */
    struct {			/* IP options (if any) */
      u_char code;
      u_char len;
      u_char data[40];		/* the maximum possible length */
    } opts[40];			/* maximum possible amount */
    int opt_count;
  } ip;

  int is_tcp;
  struct tcp_inf {
    u_short sp;			/* source port */
    u_short dp;			/* dest. port */
    u_long seq;			/* sequence number */
    u_long ack;			/* acknoledgement number */
    u_int off;		        /* offset */
    u_int x2;                   /* reserved */
    u_char flags;		/* flags */
    u_short win;
    u_short csum;
    u_short urg;
    struct {			/* IP options (if any) */
      u_char code;
      u_char len;
      u_char data[40];		/* the maximum possible length */
    } opts[40];			/* maximum possible amount */
      int opt_count;
  } tcp;

  int is_udp;
  struct udp_inf {
    u_short sp;			/* source port */
    u_short dp;			/* dest. port */
    u_short len;		/* UDP datagram length */
    u_short csum;		/* Checksum */
  } udp;

  int is_icmp;
  struct icmp_inf {
    u_char type;
    u_char code;
    u_short csum;
    u_short id;
    u_short seq;
    int ext;			/* extended ICMP info */
  } icmp;

  char *payload;
  int plen;			/* payload length */

} packet_info_t;

int decode_packet(packet_info_t *, struct pcap_pkthdr *, u_char *);
void print_payload(packet_info_t *);

#endif
