/*
   mp_spoolman.h

   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems
   Copyright(C) 2004,2005 Jonathan Le Plastrier, Farm9.com
   Copyright(C) 2005 John P. Silva, Farm9.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: mp_spoolman.h,v 1.5 2005/09/02 00:01:25 jsilva Exp $ */

#ifndef _MP_SPOOLMAN_H_
#define _MP_SPOOLMAN_H_

#include <sys/param.h>
#include <sys/types.h>
#include "mp_output.h"

/* ===================================================== */
typedef struct			/* spool file descriptor */
{
  int fd;			/* right */
  off_t offset;			/* current offset */
  u_int record;			/* number of records processed */
  char filepath[MAXPATHLEN];	/* goes to the checkpoint file too */
  time_t timet;			/* snort's extension */
  ino_t inode;			/* from stat() */
  off_t size;			/* from stat() */
} spoolfile_t;

typedef struct _out_mod {
  char *name;
  char *params;

  /* functions exported from output module */
  mp_out_log_t *out_log;
  mp_out_alert_t *out_alert;
  mp_out_fini_t *out_fini;

  struct _out_mod *next;

} out_mod_t;

typedef struct /* the main spool structure - the mix of configurable and run-time parameters */
{
#define AFILE 0
#define LFILE 1

  spoolfile_t file[2];		/* alert and log */

  uid_t uid; /* uid/gid for the spool processor (works if parent's uid = 0) */
  gid_t gid;

  int delete_processed;
  char *arch_dir;
  char *spool_dir;
  char *check_file;		/* checkpoint filename */
  char *lock;
  out_mod_t *o_mods;		/* output module chain */

  char *sid_file;		/* sid filename */
  char *gen_file;		/* gen filename */
  char *class_file;		/* class filename */
  char *urlhead_file;		/* urlhead filename */

  int restarts;	  /* how many times has this spool been (re)started */

  /* some shortcuts... */
# define alert_record file[AFILE].record
# define log_record file[LFILE].record

# define alert_offset file[AFILE].offset
# define log_offset file[LFILE].offset

# define alert_filepath file[AFILE].filepath
# define log_filepath file[LFILE].filepath

# define alert_fd file[AFILE].fd
# define log_fd file[LFILE].fd

} spool_t;

/* allocated in spool.c, configured from configparse.c */
extern spool_t *SPOOLS;
extern int SPOOLS_NUMBER;

int process_spools(void);
void cleanup_spools(void);

#endif
