/*
   mp_util.h

   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: mp_util.h,v 1.2 2004/12/11 21:15:55 jleplast Exp $ */

#ifndef _MP_UTIL_H_
#define _MP_UTIL_H_

#include <sys/time.h>

/* exit code for inrecoverable fatal error in child process */
#define CHILD_FATAL 123

int spool_lock(char *name);
void spool_unlock(int fd);

void reset_sigmask(void);

void check_uids(void);

pid_t mypid(void);
void mypid_set(void);

void check_file(char *name, int full_access);
void check_dir(char *dir, int full_access);

void tv_copy(struct timeval *dst, struct timeval *src);
void tv_zero(struct timeval *one);
int tv_is_null(struct timeval *one);
int tv_compare(struct timeval *one, struct timeval *two);
double tv_diff(struct timeval *one, struct timeval *two);

/* verbosity levels */
#define V_norm 0
#define V_gen  1
#define V_func 2
#define V_all  3
#define V_verb 4		/* even more than all */

void debug(int level, const char *, ...);
void logerr(char *op, char *f);
void logprint(const char *, ...);
void fatal(const char *, ...);
void child_fatal(const char *, ...); /* do not kill parent process */

void *array_alloc(void *pointer, int index, int elsize, int chunk_size);
void *my_alloc(unsigned long);

char *remove_space(char *);

void go_daemon();
int close_pidfile();

void my_exit(void);

void my_sleep(void);

#endif
