/*
   mudpit.c

   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems
   Copyright(C) 2004,2005 Jonathan Le Plastrier, Farm9.com

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/* $Id: mudpit.c,v 1.4 2005/06/22 21:12:29 jleplast Exp $ */

#include "config.h"

#include <signal.h>
#include <setjmp.h>
#include <unistd.h>
#include <syslog.h>

#include "mudpit.h"
#include "mp_spoolman.h"
#include "mp_util.h"
#include "mp_conf.h"
#include "mp_maps.h"

global_conf_t GCF;

/* return codes for longjmp() */
#define ACTION_RECONFIG 1
#define ACTION_EXIT 2

static jmp_buf jmpbuf;

static void sighandler(int sig)
{
  register int act = 0;
  
  switch(sig) {
  case SIGHUP:
    act = ACTION_RECONFIG;
    break;
  case SIGCHLD: case SIGALRM:
    break;
  default:
    act = ACTION_EXIT;
    break;
  }

  /* POSIX thing... */
  signal(sig, sighandler);

  /* jump to main(). */
  if(act) longjmp(jmpbuf, act);
}

int main(int argc, char *argv[])
{
  static int reconfig = 0;
  static int iam_daemon = 0; 
  static int sigs[] = {SIGTERM, SIGINT, SIGQUIT, SIGHUP, SIGALRM, SIGCHLD, 0};

  save_pid();
  umask(0);

  switch(setjmp(jmpbuf)) {

  case 0:
    {int i = 0; while(sigs[i]) signal(sigs[i++], sighandler);}
    logprint("%s v %s (c) 2002-2003 Fidelis Security Systems, Inc. 2004-2005 Farm9.com, Inc.\n", PACKAGE_NAME, PACKAGE_VERSION);
    check_uids();
    parse_args(argc, argv);

  case ACTION_RECONFIG:
    reset_sigmask();

    if(!reconfig) reconfig = 1;
    else {
      clear_global_lists();
      cleanup_spools();
    }

    parse_conf();

    if(GCF.verbose >= V_func) print_conf(argc, argv);

    init_global_lists();

    if(GCF.daemon_flag && !iam_daemon) {
      iam_daemon = 1;
      openlog(PACKAGE_NAME, LOG_PID, LOG_DAEMON);
      go_daemon();		/* new pid gets saved here */
    }

    if(process_spools());
    else logprint("Error processing spool directory\n");

    /* We are here - exiting */

  default:
    reset_sigmask();
    cleanup_spools();
    logprint("Exiting.\n");
    closelog();
  }

  return 0;
}
