/* 
   unified.h 

   Copyright (C) 1998-2002 Martin Roesch <roesch@sourcefire.com>
   Copyright(C) 2002 Gene Savchuk, Fidelis Security Systems

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

*/

/* $Id: unified.h,v 1.3 2004/12/11 21:53:03 jleplast Exp $ */

/*
 * this file contains definitions for Snort's 'unified' output.
 */

#ifndef _UNIFIED_H_
#define _UNIFIED_H_

#include <sys/types.h>

/* FARM9_BSD_PORT Added to work under FreeBSD */
#include <sys/time.h>

/* Magic numbers for unified datafiles
 */

#define ALERT_MAGIC  0xDEAD4137
#define LOG_MAGIC    0xDEAD1080

/* Snort's Event structure
 */
typedef struct _Event
{
  u_int32_t sig_generator;   /* which part of snort generated the alert? */
  u_int32_t sig_id;          /* sig id for this generator */
  u_int32_t sig_rev;         /* sig revision for this id */
  u_int32_t classification;  /* event classification */
  u_int32_t priority;        /* event priority */
  u_int32_t event_id;        /* event ID */
  u_int32_t event_reference; /* reference to other events that have gone off,
                                * such as in the case of tagged packets...
                                */
  struct timeval ref_time;   /* reference time for the event reference */
} Event;

/* ================================================== */

/* Snort's Unified Alert format
 */
typedef struct _UnifiedAlertFileHeader
{
    u_int32_t magic;
    u_int32_t version_major;
    u_int32_t version_minor;
    u_int32_t timezone;
} UnifiedAlertFileHeader;

typedef struct _UnifiedAlertRecord
{
    Event event;
    struct timeval ts;         /* event timestamp */
    u_int32_t sip;             /* src ip */
    u_int32_t dip;             /* dest ip */
    u_int16_t sp;              /* src port */
    u_int16_t dp;              /* dest port */
    u_int32_t protocol;        /* protocol id */
    u_int32_t flags;           /* any other flags (fragmented, etc) */
} UnifiedAlertRecord;

/* ================================================== */

/* Snort's Unified Log format
 */
typedef struct _UnifiedLogFileHeader
{
    u_int32_t magic;
    u_int16_t version_major;
    u_int16_t version_minor;
    u_int32_t timezone;
    u_int32_t sigfigs;
    u_int32_t snaplen;
    u_int32_t linktype;
} UnifiedLogFileHeader;

typedef struct
{
    struct timeval ts;     /* packet timestamp */
    u_int32_t caplen;      /* packet captured length */
    u_int32_t pktlen;      /* packet "real" length */
} PcapHdr;

typedef struct _UnifiedLog
{
    Event event;
    u_int32_t flags;
    PcapHdr pkth;
} UnifiedLog;


typedef struct _UnifiedLogRecord
{
    UnifiedLog log;
    char *pkt;
} UnifiedLogRecord;

#endif
